@@ -99,7 +99,7 @@ HRESULT CUnpacker::UnpackChunk(
     if (!packBuf.Data)
       return E_OUTOFMEMORY;
     
-    RINOK(ReadStream_FALSE(inStream, packBuf.Data, inSize));
+    RINOK(ReadStream_FALSE(inStream, packBuf.Data, inSize))
 
     TotalPacked += inSize;
     
@@ -124,7 +124,7 @@ HRESULT CUnpacker::UnpackChunk(
     else
     {
       res = lzmsDecoder->Code(packBuf.Data, inSize, unpackBuf.Data, outSize);
-      unpackedSize = lzmsDecoder->GetUnpackSize();;
+      unpackedSize = lzmsDecoder->GetUnpackSize();
     }
   }
   
@@ -141,7 +141,7 @@ HRESULT CUnpacker::UnpackChunk(
   
   if (outStream)
   {
-    RINOK(WriteStream(outStream, unpackBuf.Data, outSize));
+    RINOK(WriteStream(outStream, unpackBuf.Data, outSize))
   }
   
   return res;
@@ -168,7 +168,7 @@ HRESULT CUnpacker::Unpack2(
     CMyComPtr<ISequentialInStream> limitedStream = limitedStreamSpec;
     limitedStreamSpec->SetStream(inStream);
     
-    RINOK(inStream->Seek(resource.Offset, STREAM_SEEK_SET, NULL));
+    RINOK(InStream_SeekSet(inStream, resource.Offset))
     if (resource.PackSize != resource.UnpackSize)
       return S_FALSE;
 
@@ -221,7 +221,7 @@ HRESULT CUnpacker::Unpack2(
       size_t cur = chunkSize - offsetInChunk;
       if (cur > rem)
         cur = (size_t)rem;
-      RINOK(WriteStream(outStream, unpackBuf.Data + offsetInChunk, cur));
+      RINOK(WriteStream(outStream, unpackBuf.Data + offsetInChunk, cur))
       outProcessed += cur;
       rem -= cur;
       offsetInChunk = 0;
@@ -233,20 +233,20 @@ HRESULT CUnpacker::Unpack2(
       if (rem == 0)
         return S_OK;
     
-      UInt64 offset = ss.Chunks[chunkIndex];
-      UInt64 packSize = ss.GetChunkPackSize(chunkIndex);
+      const UInt64 offset = ss.Chunks[chunkIndex];
+      const UInt64 packSize = ss.GetChunkPackSize(chunkIndex);
       const CResource &rs = db->DataStreams[ss.StreamIndex].Resource;
-      RINOK(inStream->Seek(rs.Offset + ss.HeadersSize + offset, STREAM_SEEK_SET, NULL));
+      RINOK(InStream_SeekSet(inStream, rs.Offset + ss.HeadersSize + offset))
       
       size_t cur = chunkSize;
-      UInt64 unpackRem = ss.UnpackSize - ((UInt64)chunkIndex << chunkSizeBits);
+      const UInt64 unpackRem = ss.UnpackSize - ((UInt64)chunkIndex << chunkSizeBits);
       if (cur > unpackRem)
         cur = (size_t)unpackRem;
       
       _solidIndex = -1;
       _unpackedChunkIndex = 0;
       
-      HRESULT res = UnpackChunk(inStream, ss.Method, chunkSizeBits, (size_t)packSize, cur, NULL);
+      const HRESULT res = UnpackChunk(inStream, (unsigned)ss.Method, chunkSizeBits, (size_t)packSize, cur, NULL);
       
       if (res != S_OK)
       {
@@ -266,11 +266,11 @@ HRESULT CUnpacker::Unpack2(
       if (cur > rem)
         cur = (size_t)rem;
       
-      RINOK(WriteStream(outStream, unpackBuf.Data + offsetInChunk, cur));
+      RINOK(WriteStream(outStream, unpackBuf.Data + offsetInChunk, cur))
       
       if (progress)
       {
-        RINOK(progress->SetRatioInfo(&packProcessed, &outProcessed));
+        RINOK(progress->SetRatioInfo(&packProcessed, &outProcessed))
         packProcessed += packSize;
         outProcessed += cur;
       }
@@ -311,8 +311,8 @@ HRESULT CUnpacker::Unpack2(
     if (sizesBufSize != sizesBufSize64)
       return E_OUTOFMEMORY;
     sizesBuf.AllocAtLeast(sizesBufSize);
-    RINOK(inStream->Seek(baseOffset, STREAM_SEEK_SET, NULL));
-    RINOK(ReadStream_FALSE(inStream, sizesBuf, sizesBufSize));
+    RINOK(InStream_SeekSet(inStream, baseOffset))
+    RINOK(ReadStream_FALSE(inStream, sizesBuf, sizesBufSize))
     baseOffset += sizesBufSize64;
     numChunks = (size_t)numChunks64;
   }
@@ -341,11 +341,11 @@ HRESULT CUnpacker::Unpack2(
     if (inSize != inSize64)
       return S_FALSE;
 
-    RINOK(inStream->Seek(baseOffset + offset, STREAM_SEEK_SET, NULL));
+    RINOK(InStream_SeekSet(inStream, baseOffset + offset))
 
     if (progress)
     {
-      RINOK(progress->SetRatioInfo(&offset, &outProcessed));
+      RINOK(progress->SetRatioInfo(&offset, &outProcessed))
     }
     
     size_t outSize = (size_t)1 << chunkSizeBits;
@@ -353,7 +353,7 @@ HRESULT CUnpacker::Unpack2(
     if (outSize > rem)
       outSize = (size_t)rem;
 
-    RINOK(UnpackChunk(inStream, header.GetMethod(), chunkSizeBits, inSize, outSize, outStream));
+    RINOK(UnpackChunk(inStream, header.GetMethod(), chunkSizeBits, inSize, outSize, outStream))
 
     outProcessed += outSize;
     offset = nextOffset;
@@ -494,8 +494,8 @@ void CDatabase::GetItemName(unsigned index, NWindows::NCOM::CPropVariant &name)
 void CDatabase::GetItemPath(unsigned index1, bool showImageNumber, NWindows::NCOM::CPropVariant &path) const
 {
   unsigned size = 0;
-  int index = index1;
-  int imageIndex = Items[index].ImageIndex;
+  int index = (int)index1;
+  const int imageIndex = Items[index].ImageIndex;
   const CImage &image = Images[imageIndex];
   
   unsigned newLevel = 0;
@@ -545,7 +545,7 @@ void CDatabase::GetItemPath(unsigned index1, bool showImageNumber, NWindows::NCO
   else if (needColon)
     s[0] = L':';
 
-  index = index1;
+  index = (int)index1;
   wchar_t separator = 0;
   
   for (;;)
@@ -597,7 +597,7 @@ HRESULT CDatabase::ParseDirItem(size_t pos, int parent)
     if (OpenCallback && (Items.Size() & 0xFFFF) == 0)
     {
       UInt64 numFiles = Items.Size();
-      RINOK(OpenCallback->SetCompleted(&numFiles, NULL));
+      RINOK(OpenCallback->SetCompleted(&numFiles, NULL))
     }
     
     const size_t rem = DirSize - pos;
@@ -664,7 +664,7 @@ HRESULT CDatabase::ParseDirItem(size_t pos, int parent)
       
     item.Offset = pos;
     item.Parent = parent;
-    item.ImageIndex = Images.Size() - 1;
+    item.ImageIndex = (int)Images.Size() - 1;
     
     const unsigned prevIndex = Items.Add(item);
 
@@ -677,7 +677,8 @@ HRESULT CDatabase::ParseDirItem(size_t pos, int parent)
         return S_FALSE;
       const Byte *p2 = DirData + pos;
       const UInt64 len2 = Get64(p2);
-      if ((len2 & align) != 0 || rem2 < len2 || len2 < (IsOldVersion ? 0x18 : 0x28))
+      if ((len2 & align) != 0 || rem2 < len2
+          || len2 < (unsigned)(IsOldVersion ? 0x18 : 0x28))
         return S_FALSE;
      
       DirProcessed += (size_t)len2;
@@ -742,8 +743,8 @@ HRESULT CDatabase::ParseDirItem(size_t pos, int parent)
         CItem item2;
         item2.Offset = pos;
         item2.IsAltStream = true;
-        item2.Parent = prevIndex;
-        item2.ImageIndex = Images.Size() - 1;
+        item2.Parent = (int)prevIndex;
+        item2.ImageIndex = (int)Images.Size() - 1;
         Items.Add(item2);
       }
 
@@ -775,7 +776,7 @@ HRESULT CDatabase::ParseDirItem(size_t pos, int parent)
 
     if (item.IsDir && subdirOffset != 0)
     {
-      RINOK(ParseDirItem((size_t)subdirOffset, prevIndex));
+      RINOK(ParseDirItem((size_t)subdirOffset, (int)prevIndex))
     }
   }
 }
@@ -864,7 +865,7 @@ HRESULT CDatabase::ParseImageDirs(CByteBuffer &buf, int parent)
   DirStartOffset = DirProcessed = pos;
   image.StartItem = Items.Size();
 
-  RINOK(ParseDirItem(pos, parent));
+  RINOK(ParseDirItem(pos, parent))
   
   image.NumItems = Items.Size() - image.StartItem;
   if (DirProcessed == DirSize)
@@ -899,27 +900,27 @@ HRESULT CHeader::Parse(const Byte *p, UInt64 &phySize)
     ChunkSizeBits = kChunkSizeBits;
     if (ChunkSize != 0)
     {
-      int log = GetLog(ChunkSize);
+      const int log = GetLog(ChunkSize);
       if (log < 12)
         return S_FALSE;
-      ChunkSizeBits = log;
+      ChunkSizeBits = (unsigned)log;
     }
   }
 
-  _IsOldVersion = false;
-  _IsNewVersion = false;
+  _isOldVersion = false;
+  _isNewVersion = false;
   
   if (IsSolidVersion())
-    _IsNewVersion = true;
+    _isNewVersion = true;
   else
   {
     if (Version < 0x010900)
       return S_FALSE;
-    _IsOldVersion = (Version <= 0x010A00);
+    _isOldVersion = (Version <= 0x010A00);
     // We don't know details about 1.11 version. So we use headerSize to guess exact features.
     if (Version == 0x010B00 && headerSize == 0x60)
-      _IsOldVersion = true;
-    _IsNewVersion = (Version >= 0x010D00);
+      _isOldVersion = true;
+    _isNewVersion = (Version >= 0x010D00);
   }
 
   unsigned offset;
@@ -973,7 +974,7 @@ const Byte kSignature[kSignatureSize] = { 'M', 'S', 'W', 'I', 'M', 0, 0, 0 };
 HRESULT ReadHeader(IInStream *inStream, CHeader &h, UInt64 &phySize)
 {
   Byte p[kHeaderSizeMax];
-  RINOK(ReadStream_FALSE(inStream, p, kHeaderSizeMax));
+  RINOK(ReadStream_FALSE(inStream, p, kHeaderSizeMax))
   if (memcmp(p, kSignature, kSignatureSize) != 0)
     return S_FALSE;
   return h.Parse(p, phySize);
@@ -985,7 +986,7 @@ static HRESULT ReadStreams(IInStream *inStream, const CHeader &h, CDatabase &db)
   CByteBuffer offsetBuf;
   
   CUnpacker unpacker;
-  RINOK(unpacker.UnpackData(inStream, h.OffsetResource, h, NULL, offsetBuf, NULL));
+  RINOK(unpacker.UnpackData(inStream, h.OffsetResource, h, NULL, offsetBuf, NULL))
   
   const size_t streamInfoSize = h.IsOldVersion() ? kStreamInfoSize + 2 : kStreamInfoSize;
   {
@@ -1087,7 +1088,7 @@ HRESULT CDatabase::Open(IInStream *inStream, const CHeader &h, unsigned numItems
   IsOldVersion = h.IsOldVersion();
   IsOldVersion9 = (h.Version == 0x10900);
 
-  RINOK(ReadStreams(inStream, h, *this));
+  RINOK(ReadStreams(inStream, h, *this))
 
   bool needBootMetadata = !h.MetadataResource.IsEmpty();
   unsigned numNonDeletedImages = 0;
@@ -1101,14 +1102,14 @@ HRESULT CDatabase::Open(IInStream *inStream, const CHeader &h, unsigned numItems
     if (h.PartNumber != 1 || si.PartNumber != h.PartNumber)
       continue;
 
-    const int userImage = Images.Size() + GetStartImageIndex();
+    const unsigned userImage = Images.Size() + GetStartImageIndex();
     CImage &image = Images.AddNew();
     SetRootNames(image, userImage);
     
     CByteBuffer &metadata = image.Meta;
     Byte hash[kHashSize];
     
-    RINOK(unpacker.UnpackData(inStream, si.Resource, h, this, metadata, hash));
+    RINOK(unpacker.UnpackData(inStream, si.Resource, h, this, metadata, hash))
    
     if (memcmp(hash, si.Hash, kHashSize) != 0 &&
         !(h.IsOldVersion() && IsEmptySha(si.Hash)))
@@ -1119,7 +1120,7 @@ HRESULT CDatabase::Open(IInStream *inStream, const CHeader &h, unsigned numItems
     if (Items.IsEmpty())
       Items.ClearAndReserve(numItemsReserve);
 
-    RINOK(ParseImageDirs(metadata, -1));
+    RINOK(ParseImageDirs(metadata, -1))
     
     if (needBootMetadata)
     {
@@ -1166,12 +1167,12 @@ bool CDatabase::ItemHasStream(const CItem &item) const
 }
 
 
-#define RINOZ(x) { int __tt = (x); if (__tt != 0) return __tt; }
+#define RINOZ(x) { int _tt_ = (x); if (_tt_ != 0) return _tt_; }
 
 static int CompareStreamsByPos(const CStreamInfo *p1, const CStreamInfo *p2, void * /* param */)
 {
-  RINOZ(MyCompare(p1->PartNumber, p2->PartNumber));
-  RINOZ(MyCompare(p1->Resource.Offset, p2->Resource.Offset));
+  RINOZ(MyCompare(p1->PartNumber, p2->PartNumber))
+  RINOZ(MyCompare(p1->Resource.Offset, p2->Resource.Offset))
   return MyCompare(p1->Resource.PackSize, p2->Resource.PackSize);
 }
 
@@ -1192,11 +1193,11 @@ static int FindId(const CStreamInfo *streams, const CUIntVector &sorted, UInt32
   unsigned left = 0, right = sorted.Size();
   while (left != right)
   {
-    unsigned mid = (left + right) / 2;
-    unsigned streamIndex = sorted[mid];
-    UInt32 id2 = streams[streamIndex].Id;
+    const unsigned mid = (left + right) / 2;
+    const unsigned streamIndex = sorted[mid];
+    const UInt32 id2 = streams[streamIndex].Id;
     if (id == id2)
-      return streamIndex;
+      return (int)streamIndex;
     if (id < id2)
       right = mid;
     else
@@ -1210,15 +1211,15 @@ static int FindHash(const CStreamInfo *streams, const CUIntVector &sorted, const
   unsigned left = 0, right = sorted.Size();
   while (left != right)
   {
-    unsigned mid = (left + right) / 2;
-    unsigned streamIndex = sorted[mid];
+    const unsigned mid = (left + right) / 2;
+    const unsigned streamIndex = sorted[mid];
     const Byte *hash2 = streams[streamIndex].Hash;
     unsigned i;
     for (i = 0; i < kHashSize; i++)
       if (hash[i] != hash2[i])
         break;
     if (i == kHashSize)
-      return streamIndex;
+      return (int)streamIndex;
     if (hash[i] < hash2[i])
       right = mid;
     else
@@ -1237,8 +1238,8 @@ static int CompareItems(const unsigned *a1, const unsigned *a2, void *param)
     return i1.IsDir ? -1 : 1;
   if (i1.IsAltStream != i2.IsAltStream)
     return i1.IsAltStream ? 1 : -1;
-  RINOZ(MyCompare(i1.StreamIndex, i2.StreamIndex));
-  RINOZ(MyCompare(i1.ImageIndex, i2.ImageIndex));
+  RINOZ(MyCompare(i1.StreamIndex, i2.StreamIndex))
+  RINOZ(MyCompare(i1.ImageIndex, i2.ImageIndex))
   return MyCompare(i1.Offset, i2.Offset);
 }
 
@@ -1286,7 +1287,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
         if (si.RefCount != 1)
           return S_FALSE;
 
-        r.SolidIndex = Solids.Size();
+        r.SolidIndex = (int)Solids.Size();
 
         CSolid &ss = Solids.AddNew();
         ss.StreamIndex = k;
@@ -1300,8 +1301,8 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
 
           const CVolume &vol = volumes[si.PartNumber];
           IInStream *inStream = vol.Stream;
-          RINOK(inStream->Seek(r.Offset, STREAM_SEEK_SET, NULL));
-          RINOK(ReadStream_FALSE(inStream, (Byte *)header, kSolidHeaderSize));
+          RINOK(InStream_SeekSet(inStream, r.Offset))
+          RINOK(ReadStream_FALSE(inStream, (Byte *)header, kSolidHeaderSize))
           
           ss.UnpackSize = GetUi64(header);
 
@@ -1313,11 +1314,11 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
             return S_FALSE;
 
           const UInt32 solidChunkSize = GetUi32(header + 8);
-          int log = GetLog(solidChunkSize);
+          const int log = GetLog(solidChunkSize);
           if (log < 8 || log > 31)
             return S_FALSE;
-          ss.ChunkSizeBits = log;
-          ss.Method = GetUi32(header + 12);
+          ss.ChunkSizeBits = (unsigned)log;
+          ss.Method = (Int32)GetUi32(header + 12);
           
           UInt64 numChunks64 = (ss.UnpackSize + (((UInt32)1 << ss.ChunkSizeBits) - 1)) >> ss.ChunkSizeBits;
           UInt64 sizesBufSize64 = 4 * numChunks64;
@@ -1327,7 +1328,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
             return E_OUTOFMEMORY;
           sizesBuf.AllocAtLeast(sizesBufSize);
           
-          RINOK(ReadStream_FALSE(inStream, sizesBuf, sizesBufSize));
+          RINOK(ReadStream_FALSE(inStream, sizesBuf, sizesBufSize))
           
           size_t numChunks = (size_t)numChunks64;
           ss.Chunks.Alloc(numChunks + 1);
@@ -1381,14 +1382,14 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
         CSolid &ss = Solids[solidIndex];
         if (r.Offset < ss.SolidOffset)
           return S_FALSE;
-        UInt64 relat = r.Offset - ss.SolidOffset;
+        const UInt64 relat = r.Offset - ss.SolidOffset;
         if (relat > ss.UnpackSize)
           return S_FALSE;
         if (r.PackSize > ss.UnpackSize - relat)
           return S_FALSE;
-        r.SolidIndex = solidIndex;
+        r.SolidIndex = (int)solidIndex;
         if (ss.FirstSmallStream < 0)
-          ss.FirstSmallStream = k;
+          ss.FirstSmallStream = (int)k;
 
         sortedByHash.AddInReserved(k);
         // ss.NumRefs++;
@@ -1542,7 +1543,7 @@ HRESULT CDatabase::FillAndCheck(const CObjectVector<CVolume> &volumes)
         {
           CItem item;
           item.Offset = 0;
-          item.StreamIndex = i;
+          item.StreamIndex = (int)i;
           item.ImageIndex = -1;
           Items.Add(item);
           ThereAreDeletedStreams = true;
@@ -1591,7 +1592,7 @@ HRESULT CDatabase::GenerateSortedItems(int imageIndex, bool showImageNumber)
   
   if (NumExcludededItems != 0)
   {
-    ExludedItem = startItem;
+    ExludedItem = (int)startItem;
     startItem += NumExcludededItems;
   }
 
@@ -1603,7 +1604,7 @@ HRESULT CDatabase::GenerateSortedItems(int imageIndex, bool showImageNumber)
 
   SortedItems.Sort(CompareItems, this);
   for (i = 0; i < SortedItems.Size(); i++)
-    Items[SortedItems[i]].IndexInSorted = i;
+    Items[SortedItems[i]].IndexInSorted = (int)i;
 
   if (showImageNumber)
     for (i = 0; i < Images.Size(); i++)
@@ -1611,7 +1612,7 @@ HRESULT CDatabase::GenerateSortedItems(int imageIndex, bool showImageNumber)
       CImage &image = Images[i];
       if (image.NumEmptyRootItems != 0)
         continue;
-      image.VirtualRootIndex = VirtualRoots.Size();
+      image.VirtualRootIndex = (int)VirtualRoots.Size();
       VirtualRoots.Add(i);
     }
 
@@ -1681,7 +1682,7 @@ HRESULT CDatabase::ExtractReparseStreams(const CObjectVector<CVolume> &volumes,
       if ((unpacker.TotalPacked - totalPackedPrev) >= ((UInt32)1 << 16))
       {
         UInt64 numFiles = Items.Size();
-        RINOK(openCallback->SetCompleted(&numFiles, &unpacker.TotalPacked));
+        RINOK(openCallback->SetCompleted(&numFiles, &unpacker.TotalPacked))
         totalPackedPrev = unpacker.TotalPacked;
       }
     }
@@ -1715,7 +1716,7 @@ HRESULT CDatabase::ExtractReparseStreams(const CObjectVector<CVolume> &volumes,
       if (res == S_FALSE)
         continue;
 
-      RINOK(res);
+      RINOK(res)
       
       if (memcmp(digest, si.Hash, kHashSize) != 0
         // && !(h.IsOldVersion() && IsEmptySha(si.Hash))
@@ -1729,11 +1730,11 @@ HRESULT CDatabase::ExtractReparseStreams(const CObjectVector<CVolume> &volumes,
     CByteBuffer &reparse = ReparseItems.AddNew();
     reparse.Alloc(8 + buf.Size());
     Byte *dest = (Byte *)reparse;
-    SetUi32(dest, tag);
-    SetUi32(dest + 4, (UInt32)buf.Size());
+    SetUi32(dest, tag)
+    SetUi32(dest + 4, (UInt32)buf.Size())
     if (buf.Size() != 0)
       memcpy(dest + 8, buf, buf.Size());
-    ItemToReparse[itemIndex] = ReparseItems.Size() - 1;
+    ItemToReparse[itemIndex] = (int)ReparseItems.Size() - 1;
   }
 
   return S_OK;
@@ -1856,7 +1857,7 @@ bool CWimXml::Parse()
           return false;
       }
 
-      imageInfo.ItemIndexInXml = i;
+      imageInfo.ItemIndexInXml = (int)i;
       Images.Add(imageInfo);
     }
 
