@@ -68,7 +68,7 @@ class CProcessSnapshot
 {
   HANDLE _handle;
 public:
-  CProcessSnapshot(): _handle(INVALID_HANDLE_VALUE) {};
+  CProcessSnapshot(): _handle(INVALID_HANDLE_VALUE) {}
   ~CProcessSnapshot() { Close(); }
 
   bool Close()
@@ -136,7 +136,7 @@ public:
   void SetFromExtension(const char *ext) // ext must be low case
   {
     ProgNames.Clear();
-    for (unsigned i = 0; i < ARRAY_SIZE(g_Progs); i++)
+    for (unsigned i = 0; i < Z7_ARRAY_SIZE(g_Progs); i++)
       if (FindExtProg(g_Progs[i].Ext, ext))
       {
         ProgNames.Add(g_Progs[i].Prog);
@@ -220,22 +220,25 @@ static void My_GetProcessFileName(HANDLE hProcess, UString &path)
   const unsigned maxPath = 1024;
   WCHAR temp[maxPath + 1];
   
-  const char *func_name = "GetProcessImageFileNameW";
-  Func_GetProcessImageFileNameW my_func = (Func_GetProcessImageFileNameW)
-    (void *)::GetProcAddress(::GetModuleHandleA("kernel32.dll"), func_name);
+  const char *func_name =
+      "GetProcessImageFileNameW";
+  Func_GetProcessImageFileNameW my_func = Z7_GET_PROC_ADDRESS(
+  Func_GetProcessImageFileNameW, ::GetModuleHandleA("kernel32.dll"), func_name);
   
   if (!my_func)
   {
     if (!g_Psapi_dll_module)
       g_Psapi_dll_module = LoadLibraryW(L"Psapi.dll");
     if (g_Psapi_dll_module)
-      my_func = (Func_GetProcessImageFileNameW)(void *)::GetProcAddress(g_Psapi_dll_module, func_name);
+      my_func = Z7_GET_PROC_ADDRESS(
+        Func_GetProcessImageFileNameW, g_Psapi_dll_module, func_name);
   }
   
   if (my_func)
   {
-    // DWORD num = GetProcessImageFileNameW(hProcess, temp, maxPath);
-    DWORD num = my_func(hProcess, temp, maxPath);
+    const DWORD num =
+    // GetProcessImageFileNameW(hProcess, temp, maxPath);
+    my_func(hProcess, temp, maxPath);
     if (num != 0)
       path = temp;
   }
@@ -318,11 +321,13 @@ public:
   void SetMainProcess(HANDLE h)
   {
     #ifndef UNDER_CE
-
-    Func_GetProcessId func = (Func_GetProcessId)(void *)::GetProcAddress(::GetModuleHandleA("kernel32.dll"), "GetProcessId");
+    const
+    Func_GetProcessId func = Z7_GET_PROC_ADDRESS(
+    Func_GetProcessId, ::GetModuleHandleA("kernel32.dll"),
+        "GetProcessId");
     if (func)
     {
-      DWORD id = func(h);
+      const DWORD id = func(h);
       if (id != 0)
         _ids.AddToUniqueSorted(id);
     }
@@ -462,7 +467,7 @@ HRESULT CPanel::OpenAsArc(IInStream *inStream,
   else
   {
     if (!folderLink.FileInfo.Find(folderLink.FilePath))
-      return ::GetLastError();
+      return GetLastError_noZero_HRESULT();
     if (folderLink.FileInfo.IsDir())
       return S_FALSE;
     folderLink.IsVirtual = false;
@@ -471,14 +476,14 @@ HRESULT CPanel::OpenAsArc(IInStream *inStream,
   folderLink.VirtualPath = virtualFilePath;
 
   CFfpOpen ffp;
-  HRESULT res = ffp.OpenFileFolderPlugin(inStream,
+  const HRESULT res = ffp.OpenFileFolderPlugin(inStream,
       folderLink.FilePath.IsEmpty() ? us2fs(virtualFilePath) : folderLink.FilePath,
       arcFormat, GetParent());
 
   openRes.Encrypted = ffp.Encrypted;
   openRes.ErrorMessage = ffp.ErrorMessage;
 
-  RINOK(res);
+  RINOK(res)
  
   folderLink.Password = ffp.Password;
   folderLink.UsePassword = ffp.Encrypted;
@@ -571,7 +576,7 @@ HRESULT CPanel::OpenAsArc_Name(const UString &relPath, const UString &arcFormat
 }
 
 
-HRESULT CPanel::OpenAsArc_Index(int index, const wchar_t *type
+HRESULT CPanel::OpenAsArc_Index(unsigned index, const wchar_t *type
     // , bool showErrorMessage
     )
 {
@@ -653,36 +658,13 @@ static const char * const kStartExtensions =
   " mak clw csproj vcproj sln dsp dsw"
   " ";
 
-static bool FindExt(const char *p, const UString &name)
-{
-  int dotPos = name.ReverseFind_Dot();
-  if (dotPos < 0 || dotPos == (int)name.Len() - 1)
-    return false;
-
-  AString s;
-  for (unsigned pos = dotPos + 1;; pos++)
-  {
-    wchar_t c = name[pos];
-    if (c == 0)
-      break;
-    if (c >= 0x80)
-      return false;
-    s += (char)MyCharLower_Ascii((char)c);
-  }
-  for (unsigned i = 0; p[i] != 0;)
-  {
-    unsigned j;
-    for (j = i; p[j] != ' '; j++);
-    if (s.Len() == j - i && memcmp(p + i, (const char *)s, s.Len()) == 0)
-      return true;
-    i = j + 1;
-  }
-  return false;
-}
+// bool FindExt(const char *p, const UString &name, AString &s);
+bool FindExt(const char *p, const UString &name, CStringFinder &finder);
 
 static bool DoItemAlwaysStart(const UString &name)
 {
-  return FindExt(kStartExtensions, name);
+  CStringFinder finder;
+  return FindExt(kStartExtensions, name, finder);
 }
 
 UString GetQuotedString(const UString &s);
@@ -754,10 +736,10 @@ static HRESULT StartEditApplication(const UString &path, bool useEditor, HWND wi
   UStringVector params;
   params.Add(path);
 
-  HRESULT res = StartAppWithParams(command, params, process);
-  if (res != SZ_OK)
+  const WRes res = StartAppWithParams(command, params, process);
+  if (res != 0)
     ::MessageBoxW(window, LangString(IDS_CANNOT_START_EDITOR), L"7-Zip", MB_OK  | MB_ICONSTOP);
-  return res;
+  return HRESULT_FROM_WIN32(res);
 }
 
 
@@ -772,7 +754,7 @@ void CApp::DiffFiles()
   }
 
   CRecordVector<UInt32> indices;
-  panel.GetSelectedItemsIndices(indices);
+  panel.Get_ItemIndices_Selected(indices);
 
   UString path1, path2;
   if (indices.Size() == 2)
@@ -792,7 +774,7 @@ void CApp::DiffFiles()
 
     path1 = panel.GetItemFullPath(indices[0]);
     CRecordVector<UInt32> indices2;
-    destPanel.GetSelectedItemsIndices(indices2);
+    destPanel.Get_ItemIndices_Selected(indices2);
     if (indices2.Size() == 1)
       path2 = destPanel.GetItemFullPath(indices2[0]);
     else
@@ -820,19 +802,19 @@ void CApp::DiffFiles(const UString &path1, const UString &path2)
   params.Add(path1);
   params.Add(path2);
 
-  HRESULT res;
+  WRes res;
   {
     CProcess process;
     res = StartAppWithParams(command, params, process);
   }
-  if (res == SZ_OK)
+  if (res == 0)
     return;
   ::MessageBoxW(_window, LangString(IDS_CANNOT_START_EDITOR), L"7-Zip", MB_OK  | MB_ICONSTOP);
 }
 
 
 #ifndef _UNICODE
-typedef BOOL (WINAPI * ShellExecuteExWP)(LPSHELLEXECUTEINFOW lpExecInfo);
+typedef BOOL (WINAPI * Func_ShellExecuteExW)(LPSHELLEXECUTEINFOW lpExecInfo);
 #endif
 
 static HRESULT StartApplication(const UString &dir, const UString &path, HWND window, CProcess &process)
@@ -844,7 +826,7 @@ static HRESULT StartApplication(const UString &dir, const UString &path, HWND wi
     int dot = path2.ReverseFind_Dot();
     int separ = path2.ReverseFind_PathSepar();
     if (dot < 0 || dot < separ)
-      path2 += '.';
+      path2.Add_Dot();
   }
   #endif
 
@@ -862,12 +844,15 @@ static HRESULT StartApplication(const UString &dir, const UString &path, HWND wi
     execInfo.lpParameters = NULL;
     execInfo.lpDirectory = dir.IsEmpty() ? NULL : (LPCWSTR)dir;
     execInfo.nShow = SW_SHOWNORMAL;
-    execInfo.hProcess = 0;
-    ShellExecuteExWP shellExecuteExW = (ShellExecuteExWP)
-    (void *)::GetProcAddress(::GetModuleHandleW(L"shell32.dll"), "ShellExecuteExW");
-    if (!shellExecuteExW)
+    execInfo.hProcess = NULL;
+    const
+    Func_ShellExecuteExW
+       f_ShellExecuteExW = Z7_GET_PROC_ADDRESS(
+    Func_ShellExecuteExW, ::GetModuleHandleW(L"shell32.dll"),
+        "ShellExecuteExW");
+    if (!f_ShellExecuteExW)
       return 0;
-    shellExecuteExW(&execInfo);
+    f_ShellExecuteExW(&execInfo);
     result = (UINT32)(UINT_PTR)execInfo.hInstApp;
     process.Attach(execInfo.hProcess);
   }
@@ -895,7 +880,7 @@ static HRESULT StartApplication(const UString &dir, const UString &path, HWND wi
       #endif
       ;
     execInfo.nShow = SW_SHOWNORMAL;
-    execInfo.hProcess = 0;
+    execInfo.hProcess = NULL;
     ::ShellExecuteEx(&execInfo);
     result = (UINT32)(UINT_PTR)execInfo.hInstApp;
     process.Attach(execInfo.hProcess);
@@ -927,7 +912,7 @@ static void StartApplicationDontWait(const UString &dir, const UString &path, HW
   StartApplication(dir, path, window, process);
 }
 
-void CPanel::EditItem(int index, bool useEditor)
+void CPanel::EditItem(unsigned index, bool useEditor)
 {
   if (!_parentFolders.IsEmpty())
   {
@@ -939,7 +924,7 @@ void CPanel::EditItem(int index, bool useEditor)
 }
 
 
-void CPanel::OpenFolderExternal(int index)
+void CPanel::OpenFolderExternal(unsigned index)
 {
   UString prefix = GetFsPath();
   UString path = prefix;
@@ -1010,9 +995,10 @@ bool CPanel::IsVirus_Message(const UString &name)
     }
     if (i != name2.Len())
     {
+      CStringFinder finder;
       UString name3 = name2;
       name3.DeleteFrom(i);
-      if (FindExt(kExeExtensions, name3))
+      if (FindExt(kExeExtensions, name3, finder))
         isVirus = true;
     }
   }
@@ -1025,13 +1011,13 @@ bool CPanel::IsVirus_Message(const UString &name)
   
   if (!isSpaceError)
   {
-    int pos1 = s.Find(L'(');
+    const int pos1 = s.Find(L'(');
     if (pos1 >= 0)
     {
-      int pos2 = s.Find(L')', pos1 + 1);
+      const int pos2 = s.Find(L')', (unsigned)pos1 + 1);
       if (pos2 >= 0)
       {
-        s.Delete(pos1, pos2 + 1 - pos1);
+        s.Delete((unsigned)pos1, (unsigned)pos2 + 1 - (unsigned)pos1);
         if (pos1 > 0 && s[pos1 - 1] == ' ' && s[pos1] == '.')
           s.Delete(pos1 - 1);
       }
@@ -1050,10 +1036,10 @@ bool CPanel::IsVirus_Message(const UString &name)
 }
 
 
-void CPanel::OpenItem(int index, bool tryInternal, bool tryExternal, const wchar_t *type)
+void CPanel::OpenItem(unsigned index, bool tryInternal, bool tryExternal, const wchar_t *type)
 {
   CDisableTimerProcessing disableTimerProcessing(*this);
-  UString name = GetItemRelPath2(index);
+  const UString name = GetItemRelPath2(index);
   
   if (tryExternal)
     if (IsVirus_Message(name))
@@ -1096,7 +1082,7 @@ void CPanel::OpenItem(int index, bool tryInternal, bool tryExternal, const wchar
 
 class CThreadCopyFrom: public CProgressThreadVirt
 {
-  HRESULT ProcessVirt();
+  HRESULT ProcessVirt() Z7_override;
 public:
   UString FullPath;
   UInt32 ItemIndex;
@@ -1133,7 +1119,7 @@ HRESULT CPanel::OnOpenItemChanged(UInt32 index, const wchar_t *fullFilePath,
   t.UpdateCallbackSpec->Password = password;
 
 
-  RINOK(t.Create(GetItemName(index), (HWND)*this));
+  RINOK(t.Create(GetItemName(index), (HWND)*this))
   return t.Result;
 }
 
@@ -1292,7 +1278,7 @@ static THREAD_FUNC_DECL MyThreadFunction(void *param)
 
       if (firstPass && indices.Size() == 1)
       {
-        DWORD curTime = GetTickCount() - startTime;
+        const DWORD curTime = GetTickCount() - startTime;
 
         /*
         if (curTime > 5 * 1000)
@@ -1312,7 +1298,7 @@ static THREAD_FUNC_DECL MyThreadFunction(void *param)
         DEBUG_PRINT_NUM(" -- firstPass -- time = ", curTime)
       }
       
-      processes.DisableWait(indices[waitResult]);
+      processes.DisableWait(indices[(unsigned)waitResult]);
     }
 
     firstPass = false;
@@ -1324,7 +1310,7 @@ static THREAD_FUNC_DECL MyThreadFunction(void *param)
   }
 
 
-  DWORD curTime = GetTickCount() - startTime;
+  const DWORD curTime = GetTickCount() - startTime;
 
   DEBUG_PRINT_NUM("after time = ", curTime)
 
@@ -1444,15 +1430,14 @@ static bool WriteZoneFile(CFSTR fileName, const CByteBuffer &buf)
 */
 
 /*
-class CBufSeqOutStream_WithFile:
-  public ISequentialOutStream,
-  public CMyUnknownImp
-{
+Z7_CLASS_IMP_COM_1(
+  CBufSeqOutStream_WithFile
+  , ISequentialOutStream
+)
   Byte *_buffer;
   size_t _size;
   size_t _pos;
 
-
   size_t _fileWritePos;
   bool fileMode;
 public:
@@ -1478,9 +1463,6 @@ public:
 
   HRESULT FlushToFile();
   size_t GetPos() const { return _pos; }
-
-  MY_UNKNOWN_IMP
-  STDMETHOD(Write)(const void *data, UInt32 size, UInt32 *processedSize);
 };
 
 static const UInt32 kBlockSize = ((UInt32)1 << 31);
@@ -1565,7 +1547,7 @@ tryInternal tryExternal
                       alwaysStart(name)  : external
 */
 
-void CPanel::OpenItemInArchive(int index, bool tryInternal, bool tryExternal, bool editMode, bool useEditor, const wchar_t *type)
+void CPanel::OpenItemInArchive(unsigned index, bool tryInternal, bool tryExternal, bool editMode, bool useEditor, const wchar_t *type)
 {
   // we don't want to change hash data here
   if (IsHashFolder())
@@ -1830,7 +1812,7 @@ void CPanel::OpenItemInArchive(int index, bool tryInternal, bool tryExternal, bo
     // win7 / win10 work so for some extensions (pdf, html ..);
     DEBUG_PRINT("#### (HANDLE)process == 0");
     // return;
-    if (res != SZ_OK)
+    if (res != S_OK)
       return;
   }
 
@@ -1839,7 +1821,7 @@ void CPanel::OpenItemInArchive(int index, bool tryInternal, bool tryExternal, bo
   tpi->FileIndex = index;
   tpi->RelPath = relPath;
   
-  if ((HANDLE)process != 0)
+  if ((HANDLE)process)
     tpi->Processes.SetMainProcess(process.Detach());
 
   ::CThread th;
